# Define cumulant classes and define  a familytoo class.
# Adds the first four cumulants to the existing family class.


cumulants <- function (family = gaussian,...){
family <- as.family(family)
famname <- family$family[1]
name.int <- charmatch(famname, c("Gaussian","Binomial", "Poisson", "Gamma"))
	if(is.na(name.int))
		stop("Invalid family name")
	else if(name.int == 0)
		stop("Ambiguous family name")

	funlist <- switch(name.int,

            list(function(mu, ...){mu},
                 function(mu, ...){
                   if(is.matrix(mu)){return(matrix(1,nrow = nrow(mu),
ncol = ncol(mu)))} else {return(rep(1,length(mu)))}},        
                 function(mu, ...){if(as.matrix(mu)){return(matrix(0,nrow = nrow(mu),
ncol = ncol(mu)))} else {return(rep(0,length(mu)))}},
                 function(mu, ...1){if(as.matrix(mu)){return(matrix(0,nrow = nrow(mu),
ncol = ncol(mu)))} else {return(rep(0,length(mu)))}}),

            list(function(mu, n =1){n * mu},
                 function(mu, n =1){n * mu * (1 - mu)},        
                 function(mu, n =1){n * mu * (1 - mu) * (1 - 2 * mu)},
                 function(mu, n =1){n * mu * (1 - mu) * (1 - (6 * mu)*(1-mu))}),
            list(function(mu,...){mu},
                 function(mu,...){mu},
                 function(mu,...){mu},
                 function(mu,...){mu}) ,

            list(function(mu,...){mu},
                 function(mu,...){mu^2},
                 function(mu,...){2 * mu^3},
                 function(mu,...){6 * mu^4 })
)

	textlist <- switch(name.int,

            list("mu",
                 "1",        
                 "0",
                 "0"),
            list("n * mu",
                 "n * mu(1 - mu)",        
                 "n * mu(1 - mu)(1 - 2 mu)",
                 "n * mu(1 - mu)(1 - 6 mu(1 - mu))"),
            list("mu",
                 "mu",
                 "mu",
                 "mu"),
            list("mu",
                 "mu^2",
                 "2 * mu^3",
                 "6 * mu^4") 
)

result <- append(textlist,funlist)

class(result) <- c("cumulants")
names(result) <- c("cone","ctwo","cthree","cfour","First","Second","Third","Fourth") 
return(result)
                 
}


print.cumulants <- function(x,...){
cat("   first cumulant:  ",x$cone)
cat("\n  second cumulant:  ",x$ctwo)
cat("\n   third cumulant:  ",x$cthree)
cat("\n  fourth cumulant:  ",x$cfour,"\n")
}


familytoo <- function(y = gaussian,...){
y <- append(family(y),cumulants(y))
class(y) <- c("familytoo","family")
return(y)
}


print.familytoo <- function(x,...){
    class(x) <- c("family")
    print(x)
    class(x) <- c("cumulants")
    print(x)
}

printhtml <- function(x,...){
UseMethod("printhtml")}

printhtml.default <- function(x,...){
print("Hit the default printhtml method")
}

printhtml.matrix <- function(x, ...){
class(x) <- NULL
rows <- dim(x)[[1]]
cols <- dim(x)[[2]]
cat("\n <CENTER><TABLE BORDER=2 CELLPADDING=2 CELLSPACING=1> \n")
cat("<TR> <TD>Name ")
for(i in 1:cols){
cat("\n<TD>",dimnames(x)[[2]][i])}

for(i in 1:rows){
cat("<TR><TD>",dimnames(x)[[1]][i])
for (j in 1:cols){
cat("<TD>", x[i,j])}
}
cat("\n</TABLE></CENTER>")
invisible(x)
}


print.psglm <- function(object, ...){
        if(!is.null(cl <- object$call)) {
                cat("Call:\n")
                dput(cl)
        }
        coef <- object$coef
        if(any(nas <- is.na(coef))) {
                if(is.null(names(coef))) names(coef) <- paste("b",
1:length(
                                coef), sep = "")        #coef <- coef[!nas]
                cat("\nCoefficients: (", sum(nas), 
                        " not defined because of singularities)\n",
sep = "")
        }
        else cat("\nCoefficients:\n")
        print(coef, ...)

cat("\n Number of  strata: ",object$strata,"\n")
cat(" Number of columns: ",object$columns,"\n")
}


summary.psglm <- function(object,correlation = F){
 
wt <- object$weights


coef <- object$coef
cnames <- names(coef)
st.dev <- sqrt(-diag(solve(object$info)))
z.val <- coef/st.dev
p.val <-  2* (1 - pnorm(abs(z.val)))

coef <- cbind(coef,st.dev,z.val,p.val)
dimnames(coef) <- list(cnames,c("Value","Std. Error", "z value", "p value"))

covun <- -solve(object$info)

        if(correlation) {
                correl <- covun /outer(sqrt(diag(covun)),sqrt(diag(covun)))
                dimnames(correl) <- list(cnames, cnames)
        } 
        else correl <- NULL 

structure(list(call = object$call, 
coefficients = coef, 
correlation = correl,
iter = object$icnt,scoretest = object$scoretest, df = object$df, tstat
= object$tstat), class = "summary.psglm") 



}

print.summary.psglm <- function(x, digits = NULL, quote = T, prefix ="")
{


coef <- x$coef
correl <- x$correlation


   if(is.null(digits))
                digits <- options()$digits
        else {
                old.digits <- options(digits = digits)
                on.exit(options(old.digits))
        }
        cat("\nCall: ")
        dput(x$call)

cat("\nCoefficients:\n")
        print(coef, digits = digits)

        cat("\nNumber of Iterations:",
format(trunc(x$iter)), 
                "\n")

        if(!is.null(correl)) {
                p <- dim(correl)[2]
                if(p > 1) {
                        cat("\nCorrelation of Coefficients:\n")
                        ll <- lower.tri(correl)
                        correl[ll] <- format(round(correl[ll],
digits))
                        correl[!ll] <- ""
                        print(correl[-1,  - p, drop = F], quote = F,
digits = 
                                digits)
                }
        }

if(!is.null(x$tstat)) {
cat("\nScore test: \n")
cat("Parameters tested equate to column(s)",
x$scoretest ,"of the design matrix\n\n") 
testresults <- c(x$tstat,x$df,1 - pchisq(x$tstat, x$df))
names(testresults) <- c("chi-sq val.", "  df", "p value")
print(testresults,digits=digits)

} 
        invisible(NULL)

}



printhtml.psglm <- function(object,...){

class(object) <- NULL

cat("\n<H3> Model fitting summary</H3>\n")
cat("Data set has ",object$strata,"strata\n<BR>")
cat("Data set has ",object$columns,"columns\n<BR>")
cat("Convergence occurred in ",object$icnt, "iterations\n<BR>")

coef <- object$coef
st.dev <- sqrt(-diag(solve(object$info)))
z.val <- coef/st.dev
p.val <-  2* (1 - pnorm(abs(z.val)))

temp <- round(cbind(coef,st.dev,z.val,p.val),3)

cat("\n<H3> Parameter estimates</H3>\n")
class(temp) <- "matrix"
printhtml(temp)

cat("\n<H3> Hypothesis test</H3>\n")

invisible(object)
}



