## Check that psglm returns the same values as glm
## when there is one stratum and conditional = F. 
## Comforting if it does.
## Also run family=gaussian through the grinder
## which is useful to ensure that other changes have not screwed
## things up.
## Check that coefficients, standard errors and correlations match.

##############
##### The basic setup
##############

options(digits=16)

rows <- 1
cols <- 40
 psi <- c(-.00,.00)
 z   <- cbind((matrix(seq(cols) - cols/2,ncol=1)) *2/cols, rnorm(cols))
 x <-
array(rep(z,rep(rows,length(z))),c(rows,cols,ncol(z)),dimnames=list(NULL,NULL,dimnames(z)[[2]]))

cat("\nStarting  Gaussian tests\n")
fam <- "gaussian"

lambda <- 3 +  -0.1 + 0 *( -abs((seq(rows) - rows/2) * 2/rows + 2))

# family to make the mean function
meanfunc <- familytoo(fam)$inverse
mu <- (if(familytoo(fam)$family[1] != "Gamma") 1 else -1) * 
meanfunc(matrix(lambda,ncol=1) %*% matrix(rep(1,cols),nrow=1)  +  
apply(x,c(2),"%*%",psi))
y <- matrix(rnorm(rows*cols,mu,10),ncol=cols)

paran <- rep(1000,4)

a1 <- summary(glm(c(y) ~ x[1,,],control=glm.control(eps=1e-16)))

dispersion <- a1$dispersion
a2 <- summary(psglm(y=y,x=x,weights=1/dispersion,paramtest=c(1,2),control=glm.control(eps=1.e-14,maxit=100),conditional=F,expinfo=T),correlation=T)
paran[1] <-  a1$coefficients[2] - a2$coefficients[1]  
paran[2] <-  a1$coefficients[2,2] - a2$coefficients[1,2]  
paran[3] <-  a1$correlation[[2,3]] - a2$correlation[[1,2]]  
paran[4] <- (a1$null.deviance - a1$deviance)/a1$dispersion - a2$tstat

if(any(abs(paran) > 1e-7)) {
cat("\nFAILED Gaussian test with conditional=F,expinfo=T")
	} else
{
cat("\nPASSED Gaussian test with conditional=F,expinfo=T\n")
}

a2 <- summary(psglm(y=y,x=x,weights=1/dispersion,paramtest=c(1,2),control=glm.control(eps=1.e-14,maxit=100),conditional=T,expinfo=T),correlation=T)

paran[1] <-  a1$coefficients[2] - a2$coefficients[1]
paran[2] <-  a1$coefficients[2,2] - a2$coefficients[1,2]
paran[3] <-  a1$correlation[[2,3]] - a2$correlation[[1,2]]
paran[4] <- (a1$null.deviance - a1$deviance)/a1$dispersion - a2$tstat


if(any(abs(paran) > 1e-7)) {
cat("\nFAILED Gaussian test with conditional=T,expinfo=T")
	} else
{
cat("\nPASSED Gaussian test with conditional=T,expinfo=T\n")
}


a2 <- summary(psglm(y=y,x=x,weights=1/dispersion,paramtest=c(1,2),control=glm.control(eps=1.e-14,maxit=100),conditional=T,expinfo=F),correlation=T)

paran[1] <-  a1$coefficients[2] - a2$coefficients[1]
paran[2] <-  a1$coefficients[2,2] - a2$coefficients[1,2]
paran[3] <-  a1$correlation[[2,3]] - a2$correlation[[1,2]]
paran[4] <- (a1$null.deviance - a1$deviance)/a1$dispersion - a2$tstat


if(any(abs(paran) > 1e-7)) {
cat("\nFAILED Gaussian test with conditional=T,expinfo=F")
	} else
{
cat("\nPASSED Gaussian test with conditional=T,expinfo=F\n")
}


########## Poisson tests.

cat("\nStarting  Poisson tests\n")
paran <- rep(1000,3)
fam <- "poisson"

lambda <- 3 +  -0.1 + 0 *( -abs((seq(rows) - rows/2) * 2/rows + 2))

# family to make the mean function
meanfunc <- familytoo(fam)$inverse
mu <- (if(familytoo(fam)$family[1] != "Gamma") 1 else -1) * 
meanfunc(matrix(lambda,ncol=1) %*% matrix(rep(1,cols),nrow=1)  +  
apply(x,c(2),"%*%",psi))

y <- matrix(rpois(rows*cols,mu),ncol=cols)

a1 <- summary(glm(c(y) ~ x[1,,],family = poisson,
              control=glm.control(eps=1e-16,maxit=100)))

a2 <- summary(psglm(y=y,x=x,family =
poisson,paramtest=c(1,2),control=glm.control(eps=1.e-14,maxit=100), 
conditional=F,expinfo=T,trace=T),correlation=T)

paran[1] <-  a1$coefficients[2] - a2$coefficients[1]  
paran[2] <-  a1$coefficients[2,2] - a2$coefficients[1,2]  
paran[3] <-  a1$correlation[[2,3]] - a2$correlation[[1,2]]  

if(any(abs(paran) > 1e-7)) {
cat("\nFAILED Poisson test with conditional=F,expinfo=T")
	} else
{
cat("\nPASSED Poisson test with conditional=F,expinfo=T\n")
}


########## Binomial tests.

cat("\nStarting  Binomial tests\n")
paran <- rep(1000,3)
fam <- "binomial"

lambda <- 0 +  -0.1 + 0 *( -abs((seq(rows) - rows/2) * 2/rows + 2))

# family to make the mean function
meanfunc <- familytoo(fam)$inverse
mu <- (if(familytoo(fam)$family[1] != "Gamma") 1 else -1) * 
meanfunc(matrix(lambda,ncol=1) %*% matrix(rep(1,cols),nrow=1)  +  
apply(x,c(2),"%*%",psi))

n <- matrix(rpois(rows*cols,5)+4,ncol=cols)
y <- matrix(rbinom(rows*cols,n,mu),ncol=cols)


a1 <- summary(glm(cbind(c(y),c(n) - c(y)) ~ x[1,,],family = binomial,
              control=glm.control(eps=1e-16,maxit=100)))

a2 <- summary(psglm(nu=n,y=y,x=x,family =
binomial,paramtest=c(1,2),control=glm.control(eps=1.e-14,maxit=100), 
conditional=F,expinfo=T,trace=T),correlation=T)

paran[1] <-  a1$coefficients[2] - a2$coefficients[1]  
paran[2] <-  a1$coefficients[2,2] - a2$coefficients[1,2]  
paran[3] <-  a1$correlation[[2,3]] - a2$correlation[[1,2]]  

if(any(abs(paran) > 1e-7)) {
cat("\nFAILED Binomial test with conditional=F,expinfo=T")
	} else
{
cat("\nPASSED Binomial test with conditional=F,expinfo=T\n")
}


########## Gamma tests.

cat("\nStarting  Gamma tests\n")
paran <- rep(1000,3)
fam <- "Gamma"

lambda <-  -0.1 + 0 *( -abs((seq(rows) - rows/2) * 2/rows + 2))

# family to make the mean function
meanfunc <- familytoo(fam)$inverse
mu <- (if(familytoo(fam)$family[1] != "Gamma") 1 else -1) * 
meanfunc(matrix(lambda,ncol=1) %*% matrix(rep(1,cols),nrow=1)  +  
apply(x,c(2),"%*%",psi))

y <- mu * matrix(rgamma(rows*cols,4),ncol=cols)

a1 <- summary(glm(c(y) ~ x[1,,],family = Gamma,
              control=glm.control(eps=1e-16,maxit=100)))

dispersion <- a1$dispersion

a2 <- summary(psglm(y=y,x=x,family =
Gamma,paramtest=c(1,2),weights=1/dispersion,
control=glm.control(eps=1.e-14,maxit=100),
conditional=F,expinfo=T,trace=T),correlation=T)

paran[1] <-  a1$coefficients[2] + a2$coefficients[1]  
paran[2] <-  a1$coefficients[2,2] - a2$coefficients[1,2]  
paran[3] <-  a1$correlation[[2,3]] - a2$correlation[[1,2]]  

if(any(abs(paran) > 1e-7)) {
cat("\nFAILED Gamma test with conditional=F,expinfo=T")
	} else
{
cat("\nPASSED Gamma test with conditional=F,expinfo=T\n")
}
